#!/usr/bin/python
#
# preferencesWindow.py
# Copyright (C) Raul E. 2008 <>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA

import pygtk
pygtk.require('2.0')
import gtk
import gtk.glade

import os
import time
import datetime

class PreferencesWindow :
    __preferences = None
    def __init__(self, parent, preferences):
        fileDirectory = os.path.dirname(__file__)
        self.glade = gtk.glade.XML(os.path.join(fileDirectory ,"glade/preferencesWindow"))
        self.glade.signal_autoconnect(self)
        self.window = self.glade.get_widget("dialog1")
        self.window.set_transient_for(parent)
        self.window.set_destroy_with_parent(True)
        self.glade.get_widget("imgColumns").set_from_pixbuf(gtk.gdk.pixbuf_new_from_file(os.path.join(fileDirectory , "../icons/column.png")))
        self.glade.get_widget("imgWidth").set_from_pixbuf(gtk.gdk.pixbuf_new_from_file(os.path.join(fileDirectory , "../icons/imagew.png")))
        self.glade.get_widget("imgHeight").set_from_pixbuf(gtk.gdk.pixbuf_new_from_file(os.path.join(fileDirectory , "../icons/imageh.png")))
        self.glade.get_widget("imgMargin").set_from_pixbuf(gtk.gdk.pixbuf_new_from_file(os.path.join(fileDirectory , "../icons/margin.png")))
        self.__preferences = preferences
    def __savePreferences(self):
        self.__preferences.setValue("Image", "columns",str(self.glade.get_widget("spnColumns").get_value_as_int()))
        self.__preferences.setValue("Image", "width", str(self.glade.get_widget("spnWidth").get_value_as_int()))
        self.__preferences.setValue("Image", "height", str(self.glade.get_widget("spnHeight").get_value_as_int()))
        self.__preferences.setValue("Image", "margin", str(self.glade.get_widget("spnMargin").get_value_as_int()))
        self.__preferences.setValue("Image", "bilinear", str(int(self.glade.get_widget("chkBilinear").get_active())))
        self.__preferences.setValue("Image", "font", self.glade.get_widget("fontButton").get_font_name())
        bgColor = self.glade.get_widget("colorBackground").get_color()
        rgbColor = str(bgColor.red) + "," + str(bgColor.green) + "," + str(bgColor.blue)
        self.__preferences.setValue("Image", "backgroundColor", rgbColor)
        self.__preferences.setValue("Image", "timeStamp", str(int(self.glade.get_widget("chkTimestamps").get_active())))
        self.__preferences.setValue("Image", "logo", str(int(self.glade.get_widget("chkLogo").get_active())))
        self.__preferences.setValue("Image", "translateHeaderText", str(int(self.glade.get_widget("chkTranslate").get_active())))
        self.__preferences.setValue("Capture", "frames", str(self.glade.get_widget("spnCaptureNum").get_value_as_int()))
        captureTime = datetime.timedelta(minutes=int(self.glade.get_widget("spnCaptureMin").get_value_as_int()), seconds=int(self.glade.get_widget("spnCaptureSec").get_value_as_int()))
        self.__preferences.setValue("Capture", "seconds", str(captureTime.seconds))
        if(self.glade.get_widget("rbCaptureSeconds").get_active() == True):
             self.__preferences.setValue("Capture", "type", "SECONDS")
        else :
             self.__preferences.setValue("Capture", "type", "FRAMES")

    def __loadPreferences(self):
        self.glade.get_widget("spnColumns").set_value(int(self.__preferences.getValue("Image", "columns")))
        self.glade.get_widget("spnWidth").set_value(int(self.__preferences.getValue("Image", "width")))
        self.glade.get_widget("spnHeight").set_value(int(self.__preferences.getValue("Image", "height")))
        self.glade.get_widget("spnMargin").set_value(int(self.__preferences.getValue("Image", "margin")))
        self.glade.get_widget("chkBilinear").set_active(bool(int(self.__preferences.getValue("Image", "bilinear"))))
        self.glade.get_widget("fontButton").set_font_name(self.__preferences.getValue("Image", "font"))
        bgColor = str(self.__preferences.getValue("Image", "backgroundColor")).split(",")
        self.glade.get_widget("colorBackground").set_color(gtk.gdk.Color(int(bgColor[0]), int(bgColor[1]), int(bgColor[2]), 1))
        self.glade.get_widget("chkTimestamps").set_active(bool(int(self.__preferences.getValue("Image", "timeStamp"))))
        self.glade.get_widget("chkLogo").set_active(bool(int(self.__preferences.getValue("Image", "logo"))))
        self.glade.get_widget("chkTranslate").set_active(bool(int(self.__preferences.getValue("Image", "translateHeaderText"))))
        self.glade.get_widget("spnCaptureNum").set_value(int(self.__preferences.getValue("Capture", "frames")))
        captureSeconds = int(self.__preferences.getValue("Capture", "seconds"))
        captureTime = time.gmtime(datetime.timedelta(seconds= captureSeconds).seconds)
        self.glade.get_widget("spnCaptureMin").set_value(int(time.strftime("%M",captureTime)))
        self.glade.get_widget("spnCaptureSec").set_value(int(time.strftime("%S",captureTime)))
        if(self.__preferences.getValue("Capture", "type").upper() == "SECONDS"):
            self.glade.get_widget("rbCaptureSeconds").set_active(True)
        else :
            self.glade.get_widget("rbCaptureNum").set_active(True)

    def on_btDefaults_clicked(self, widget):
        self.__preferences.setDefaults()
        self.window.destroy()
    def on_btOk_clicked(self, widget):
        self.__savePreferences()
        self.__preferences.save()
        self.window.destroy()
    def on_btCancel_clicked(self, widget):
        self.window.destroy()
    def show(self):
        self.__loadPreferences()
        self.window.show_all()