#!/usr/bin/python
#
# mediaLibrary.py
# Copyright (C) Raul E. 2008 <>
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA

#
# This library is based in whaaw thumbnailer source code : http://home.gna.org/whaawmp
# Missing plugin install codec functionally is inspired from elisa play_bin_engine : http://elisa.fluendo.com
#

import pygst
pygst.require("0.10")
import gst
if gst.pygst_version >= (0, 10, 10):
    import gst.pbutils

import pygtk
pygtk.require('2.0')
import gtk
import gobject

from gettext import gettext as _
import time
import datetime
import os

class CaptureType :
    SECONDS = 0
    FRAMES = 1

class MediaInfo(gobject.GObject) :
    length = 0
    width = 0
    height = 0
    codec = ""
    fileName = ""
    images = None
    errors = None
    model = None
    player = None
    captureType = CaptureType.SECONDS
    seconds = 0
    frames = 0
    __duration = 0
    __gsignals__ =  { 
            "completed": (
                gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, []),
            "progress": (
                gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, [
                gobject.TYPE_FLOAT]),
            "error": (
                gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, [
                gobject.TYPE_STRING, gobject.TYPE_STRING])
            }
    def __init__(self):
       gobject.GObject.__init__(self)
       self.size = (128, 128)
       
    def openfile(self, filename ,aModel):
       self.images = list()
       self.errors = list()
       self.fileName = filename
       self.codec = ""
       if (self.player != None):
           self.player.set_state(gst.STATE_NULL)
       self.player = gst.element_factory_make("playbin", "player")
       bin = gst.Bin("video")
       filter = gst.element_factory_make("capsfilter", "filter")
       bin.add(filter)
       filter.set_property("caps", gst.Caps("video/x-raw-rgb, depth=24, bpp=24"))
       ghostpad = gst.GhostPad("sink", filter.get_pad("sink"))
       bin.add_pad(ghostpad)
       vSink = gst.element_factory_make("fakesink", "vsink")
       bin.add(vSink)
       pad = vSink.get_pad("sink")
       pad.add_buffer_probe(self.__onBufferProbe)
       gst.element_link_many(filter, vSink)
       self.player.set_property("video-sink", bin)
       self.bus = self.player.get_bus()
       self.bus.add_signal_watch()
       self.watchID = self.bus.connect("message", self.__onMessage)
       self.player.set_property("uri", "file://" + filename)
       self.player.set_state(gst.STATE_PAUSED)
       self.model = aModel
    def __seek(self, seek_pos):
       event = self.player.seek(1.0, gst.FORMAT_TIME,
       gst.SEEK_FLAG_FLUSH | gst.SEEK_FLAG_ACCURATE,
       gst.SEEK_TYPE_SET, seek_pos,
       gst.SEEK_TYPE_NONE, 0)
       print seek_pos
    def __onBufferProbe(self, pad, buffer):
       if (True):
           caps = buffer.caps
           if (caps == None):
               print _("Stream returned no caps, exiting")
               sys.exit(6)
           filters = caps[0]
           self.width = filters["width"]
           self.height = filters["height"]
           pixbuf = gtk.gdk.pixbuf_new_from_data(buffer.data, gtk.gdk.COLORSPACE_RGB, False, 8, self.width, self.height, self.width * 3)
           self.__duration =  self.player.query_duration(gst.FORMAT_TIME)[0]/gst.SECOND
           position = self.player.query_position(gst.FORMAT_TIME)[0]/gst.SECOND
           frame = pixbuf,time.strftime("%H:%M:%S", time.gmtime(datetime.timedelta(seconds= position).seconds))
           self.images.append(frame)
           scaled = pixbuf.scale_simple(self.size[0], self.size[1], gtk.gdk.INTERP_NEAREST)
           self.model.append([scaled, frame[1]])

           if(self.captureType == CaptureType.SECONDS) :
               self.__seek((position + self.seconds) * gst.SECOND)
           else :
               if(len(self.images) < self.frames):
                   self.__seek((position + (self.__duration / (self.frames - 1))) * gst.SECOND)
               else:
                    self.__seek((self.__duration + 1) * gst.SECOND)
           percentage = float(100 * position) / float(self.__duration)
           self.emit("progress", percentage)
    def __onMessage(self, bus, message):
        if (message.type == gst.MESSAGE_ELEMENT):
            if(message.structure.get_name().startswith("missing-")):
                self.emit("completed")
                self.__getMissingElement(message)
        elif (message.type == gst.MESSAGE_ERROR):
            print message.parse_error()[0]
        elif (message.type == gst.MESSAGE_TAG):
            if(message.structure.get_name() == "video-codec"):
                self.codec = message.structure["video-codec"]
            else:
                tagList = message.parse_tag()
                keys = tagList.keys()
                try:
                    if(keys.index("video-codec") > -1 and self.codec == ""):
                        self.codec = tagList["video-codec"]
                except(ValueError):
                    None
        elif (message.type == gst.MESSAGE_ASYNC_DONE):
            self.emit("completed")
    def __getMissingElement(self, message, window_id = 0):
        if gst.pygst_version < (0, 10, 10):
            print _("This version of gstreamer can't handle missing elements")
            return
        self.errors.append(str(message.structure["type"]))
        self.errors.append(str(message.structure["detail"]))
        self.errors.append(str(message.structure["name"]))
        detail = gst.pbutils.missing_plugin_message_get_installer_detail(message)
        context = gst.pbutils.InstallPluginsContext()
    
        if window_id:
            context.set_x_id(window_id)
            
        msg = gst.pbutils.install_plugins_async([detail], context,self.__pbutils_plugin_installed_cb)
    def __pbutils_plugin_installed_cb(self, result):
        if result == gst.pbutils.INSTALL_PLUGINS_SUCCESS:
            gst.update_registry()
        else:
            self.emit("error", result.value_name, "\n".join(self.errors))
    def __getGStreamerLibraryVersion(self):
        return ".".join([str(integer) for integer in gst.pygst_version])
    def getDuration(self):
        return time.strftime("%H:%M:%S",time.gmtime(datetime.timedelta(seconds= self.__duration).seconds))
    def getDurationSeconds(self):
        return self.__duration
    def getFilename(self):
        return os.path.basename(self.fileName)
    def cancel(self):
        if (self.player != None):
           self.player.set_state(gst.STATE_NULL)
        self.player = None
        self.bus = None
        self.watchID = None

    GStreamerLibraryVersion = property(__getGStreamerLibraryVersion)
